import { CollectionReference, DocumentReference } from 'firebase/firestore/lite';
declare type CollectionPredicate<T> = string | CollectionReference<T>;
declare type DocPredicate<T> = string | DocumentReference<T>;
/**
 * Use when wanting to receive back promises that will resolve or error when internet is flaky, unlike regular firestore methods which won't resolve right away in these situations.
 * Be sure to import firestore methods such as serverTimestamp() from firebase/firestore/lite otherwise you will receive errors */
export declare function addOnline<T>(ref: CollectionPredicate<T>, data: T, opts?: {
    abbreviate?: boolean;
}): Promise<DocumentReference<T>>;
/**
 * Use when wanting to receive back promises that will resolve or error when internet is flaky, unlike regular firestore methods which won't resolve right away in these situations.
 * Be sure to import firestore methods such as serverTimestamp() from firebase/firestore/lite otherwise you will receive errors */
export declare function setOnline<T>(ref: DocPredicate<T>, data: T, opts?: {
    abbreviate?: boolean;
    merge?: boolean;
}): Promise<void>;
/**
 * Use when wanting to receive back promises that will resolve or error when internet is flaky, unlike regular firestore methods which won't resolve right away in these situations.
 * Be sure to import firestore methods such as serverTimestamp() from firebase/firestore/lite otherwise you will receive errors */
export declare function updateOnline<T>(ref: DocPredicate<T>, data: Partial<T>, opts?: {
    abbreviate?: boolean;
}): Promise<void>;
/**
 * Use when wanting to receive back promises that will resolve or error when internet is flaky, unlike regular firestore methods which won't resolve right away in these situations.
 * Be sure to import firestore methods such as serverTimestamp() from firebase/firestore/lite otherwise you will receive errors */
export declare function deleteDocumentOnline<T>(ref: DocPredicate<T>): Promise<void>;
export {};
