import { derived, writable } from 'svelte/store';
import { getAuth, onAuthStateChanged, signOut } from 'firebase/auth';
import { doc, getFirestore, serverTimestamp, updateDoc } from 'firebase/firestore';
import { firebaseAppStore } from './init';
import { docStore } from './stores';
import { setCookie } from '../helpers/cookies';
export const authState = derived(firebaseAppStore, ($firebaseApp, set) => {
    if ($firebaseApp) {
        const auth = getAuth();
        onAuthStateChanged(auth, (u) => set(u), (err) => console.error(err.message));
    }
}, null);
export function createUserStore({ userKey = 'firebase_user', log = false }) {
    const { subscribe, set } = writable(null);
    let unsub;
    let cached = null;
    cached = JSON.parse(localStorage.getItem(userKey));
    set(cached);
    authState.subscribe((u) => {
        if (u) {
            unsub && unsub();
            const userStore = docStore(`users/${u.uid}`, { log });
            unsub = userStore.subscribe((user) => {
                if (user) {
                    set(user);
                    cacheUser(user, userKey);
                    denoteVisitOnce(user);
                }
            });
        }
        else {
            set(null);
            removeCachedUser(userKey);
        }
    });
    return { subscribe };
}
export const logOut = async () => {
    const auth = getAuth();
    await signOut(auth);
};
function cacheUser(user, userKey) {
    localStorage.setItem(userKey, JSON.stringify(user));
    const minimalUser = {
        displayName: user.displayName,
        email: user.email,
        photoURL: user.photoURL || null,
    }; // Cookies are limited to 4kb, about 1,000-4000 characters
    setCookie('user', JSON.stringify(minimalUser), { 'max-age': 31536000 });
}
function removeCachedUser(userKey) {
    localStorage.removeItem(userKey);
    const yesterday = new Date(new Date());
    yesterday.setDate(yesterday.getDate() - 1);
    setCookie('user', null, { expires: yesterday });
}
const denoteVisitOnce = (() => {
    let denoted = false;
    return async function (user) {
        if (!denoted) {
            denoted = true;
            try {
                const db = getFirestore();
                await updateDoc(doc(db, 'users', user.uid), { lastVisit: serverTimestamp() });
            }
            catch (err) {
                console.error(err);
            }
            return true;
        }
        else {
            return true;
        }
    };
})();
