// must pass in firebaseApp because getFirestore() function always returns undefined when trying to fetch default app on its own accord - I think because it's designed to work in the client once Firestore has been initialized and not in one synchronous server-side operation
import { getFirestore, collection, doc, getDocs, getDoc, query, } from 'firebase/firestore';
export function colRef(ref, firebaseApp) {
    const db = getFirestore(firebaseApp);
    return typeof ref === 'string' ? collection(db, ref) : ref;
}
export function docRef(ref, firebaseApp) {
    if (typeof ref === 'string') {
        const pathParts = ref.split('/');
        const documentId = pathParts.pop();
        const collectionString = pathParts.join('/');
        return doc(colRef(collectionString, firebaseApp), documentId);
    }
    else {
        return ref;
    }
}
export async function getCollection(path, queryConstraints = [], firebaseApp) {
    const ref = typeof path === 'string' ? colRef(path, firebaseApp) : path;
    const q = query(ref, ...queryConstraints);
    const collectionSnap = await getDocs(q);
    return collectionSnap.docs.map((docSnap) => ({
        ...docSnap.data(),
        id: docSnap.id,
    }));
}
export async function getDocument(ref, firebaseApp) {
    const docSnap = await getDoc(docRef(ref, firebaseApp));
    return docSnap.exists() ? { ...docSnap.data(), id: docSnap.id } : null;
}
